from enum import Enum

class TWO_FACTOR_AUTH_METHODS(Enum):
    SMS = 0,
    PUSH = 1,
    EMAIL = 2

API_BASE_URL = 'mysecurity.eufylife.com/api'
# API_BASE_URL = 'security-app-eu.eufylife.com'
ENDPOINT_LOGIN = '/passport/login'
ENDPOINT_LOGOUT = '/passport/logout'
ENDPOINT_DEVICE_LIST = '/app/get_devs_list'
ENDPOINT_STATION_LIST = '/app/get_hub_list'

MOTION_DETECTION_COOLDOWN_MS = 3000

API_HEADERS = {
    'app_version': "v2.3.0_792",
    'os_type': "android",
    'os_version': "29",
    'phone_model': "ONEPLUS A3003",
    'country': "IT",
    'language': "en",
    'openudid': "5e4621b0152c0d00",
    'uid': "",
    'net_type': "wifi",
    'mnc': "02",
    'mcc': "262",
    'sn': "75814221ee75",
    'Model_type': "PHONE",
    'timezone': "GMT+01:00",
    'Content-Type': 'application/json'
}

class RESPONSE_ERROR_CODE(Enum):
    WHATEVER_ERROR          = 0
    SESSION_TIMEOUT         = 401
    CONNECT_ERROR           = 997
    NETWORK_ERROR           = 998
    SERVER_ERROR            = 999
    UNVERIFIED_EMAIL        = 26015
    VERIFY_CODE_ERROR       = 26050
    VERIFY_CODE_EXPIRED     = 26051
    NEED_VERIFY_CODE        = 26052
    VERIFY_CODE_MAX         = 26053
    VERIFY_CODE_NONE_MATCH  = 26054
    VERIFY_PASSWORD_ERROR   = 26055
    PHONE_NONE_SUPPORT      = 26058
    TOO_MANY_LOGIN_ATTEMPTS = 100028

class DEVICE_STATE(Enum):
    OFFLINE                 = 0
    ONLINE                  = 1
    MANUALLY_DISABLED       = 2
    OFFLINE_LOWBAT          = 3
    REMOVE_AND_READD        = 4
    RESET_AND_READD         = 5

class DEVICE_TYPE(Enum):
    UNKNOWN                 = -1
    STATION                 = 0
    CAMERA                  = 1
    SENSOR                  = 2
    FLOODLIGHT              = 3
    CAMERA_E                = 4
    DOORBELL                = 5
    BATTERY_DOORBELL        = 7
    CAMERA2C                = 8
    CAMERA2                 = 9
    MOTION_SENSOR           = 10
    KEYPAD                  = 11
    CAMERA2_PRO             = 14
    CAMERA2C_PRO            = 15
    BATTERY_DOORBELL_2      = 16
    INDOOR_CAMERA           = 30
    INDOOR_PT_CAMERA        = 31
    SOLO_CAMERA             = 32
    SOLO_CAMERA_PRO         = 33
    INDOOR_CAMERA_1080      = 34
    INDOOR_PT_CAMERA_1080   = 35
    LOCK_BASIC              = 50
    LOCK_ADVANCED           = 51
    LOCK_BASIC_NO_FINGER    = 52
    LOCK_ADVANCED_NO_FINGER = 53

class PARAM_TYPE(Enum):
    #
    PROP_EVENT_NUM          = 'event_num'
    PROP_UPDATE_TIME        = 'update_time'
    # PROP_MAIN_SW_VERSION    = 'main_sw_version'
    # PROP_MAIN_HW_VERSION    = 'main_hw_version'
    # PROP_SEC_SW_VERSION     = 'sec_sw_version'
    # PROP_SEC_HW_VERSION     = 'sec_hw_version'
    # PROP_WIFI_MAC             = 'wifi_mac'
    PROP_STATUS             = 'status'
    #
    BATTERY_LEVEL                   = 1101 #Maybe? 1177
    DEVICE_UPGRADE_NOW              = 1134
    DEVICE_RSSI                     = 1141
    
    OPEN_DEVICE                     = 2001
    NIGHT_VISUAL                    = 2002
    VOLUME                          = 2003
    DETECT_MODE                     = 2004
    DETECT_MOTION_SENSITIVE         = 2005
    DETECT_ZONE                     = 2006
    UN_DETECT_ZONE                  = 2007
    SDCARD                          = 2010
    CHIME_STATE                     = 2015
    RINGING_VOLUME                  = 2022
    DETECT_EXPOSURE                 = 2023
    DETECT_SWITCH                   = 2027
    DETECT_SCENARIO                 = 2028
    DOORBELL_HDR                    = 2029
    DOORBELL_IR_MODE                = 2030
    DOORBELL_VIDEO_QUALITY          = 2031
    DOORBELL_BRIGHTNESS             = 2032
    DOORBELL_DISTORTION             = 2033
    DOORBELL_RECORD_QUALITY         = 2034
    DOORBELL_MOTION_NOTIFICATION    = 2035
    DOORBELL_NOTIFICATION_OPEN      = 2036
    DOORBELL_SNOOZE_START_TIME      = 2037
    DOORBELL_NOTIFICATION_JUMP_MODE = 2038
    DOORBELL_LED_NIGHT_MODE         = 2039
    DOORBELL_RING_RECORD            = 2040
    DOORBELL_AUDIO_RECODE           = 2042

    WATERMARK_MODE          = 1214              # 1 - hide, 2 - show
    SNOOZE_MODE             = 1271              # The value is base64 encoded
    
    SCHEDULE_MODE           = 1257              # 0 - Away, 1 - Home, 63 - Disarmed
    GUARD_MODE              = 1224              # 0 - Away, 1 - Home, 63 - Disarmed, 2 - SCHEDULE_MODE
    
    FLOODLIGHT_MANUAL_SWITCH        = 1400
    FLOODLIGHT_MANUAL_BRIGHTNESS    = 1401         # The range is 22-100
    FLOODLIGHT_MOTION_BRIGHTNESS    = 1412         # The range is 22-100
    FLOODLIGHT_SCHEDULE_BRIGHTNESS  = 1413       # The range is 22-100
    FLOODLIGHT_MOTION_SENSITIVTY    = 1272         # The range is 1-5
    
    CAMERA_UPGRADE_NOW                  = 1133
    CAMERA_SPEAKER_VOLUME               = 1230
    CAMERA_RECORD_ENABLE_AUDIO          = 1366           # Enable microphone
    CAMERA_RECORD_RETRIGGER_INTERVAL    = 1250     # In seconds
    CAMERA_RECORD_CLIP_LENGTH           = 1249            # In seconds
    CAMERA_IR_CUT                       = 1013
    CAMERA_PIR                          = 1011
    CAMERA_WIFI_RSSI                    = 1142
    CAMERA_MOTION_ZONES                 = 1204

    SENSOR_CONTACT_OPEN                 = 1550  # 0 - Close, 1 - Open
    
    MOTION_SENSOR_ENABLE_LED            = 1607
    MOTION_SENSOR_LAST_EVENT_MS         = 1605
    MOTION_SENSOR_PIR_SENSITIVITY       = 1609
    TEST_1                              = 1608
# 
#     // Set only params?
#     PUSH_MSG_MODE = 1252,                       // 0 to ???
# }
# 


EXCLUDED_ROOT_PROPERTY = ['station_conn', 'member', 'permission', 'params', 'status']


from eufySecurityApi.const import PARAM_TYPE, DEVICE_TYPE, DEVICE_STATE, MOTION_DETECTION_COOLDOWN_MS, EXCLUDED_ROOT_PROPERTY
import logging, traceback, asyncio
from datetime import datetime, timedelta
from eufySecurityApi.utils import getUniqueId

class AttributeWrapper(dict):

    def __init__(self):
        self._internalDict = {}
    def __setitem__(self, key, value):
        if(not isinstance(key, PARAM_TYPE)):
            try:
                key = PARAM_TYPE(key)
            except:
               raise AttributeError('Key is not a PARAM_TYPE value') 
        frameStack = traceback.extract_stack(limit=2)[0]
        if(not(frameStack.filename == __file__)):
            raise AttributeError('Attribute is read only')
        self._internalDict[key] = value

    def __getitem__(self, key):
        if(isinstance(key, PARAM_TYPE)):
            return self._internalDict[key]
        else:
            return self._internalDict[PARAM_TYPE(key)]

    def __setattr__(self, key, value):
        frameStack = traceback.extract_stack(limit=2)[0]
        if(not(frameStack.name in self.__dir__() and frameStack.filename == __file__)):
            raise AttributeError('Attribute is read only')
        else:
            self.__dict__[key] = value

    def __getattr__(self, key):
        raise KeyError('Unhautorized')

    def __contains__(self, key):
        return key in self._internalDict


class callbackStruct(object):
    def __init__(self, cId, attributes, callback):
        self.cId = cId
        self.attributes = attributes
        self.call = callback

class Device(object):

    deviceType= DEVICE_TYPE(-1)
    def __init__(self, api):
        self.api = api
        self._logger  = logging.getLogger(__name__)
        # self._attribute = {}
        self._notRecognizedAttribute = {}
        self._attribute = AttributeWrapper()
    
    def __setattr__(self, key, value):
        frameStack = traceback.extract_stack(limit=2)[0]
        if(not(frameStack.filename == __file__)):
            raise AttributeError('Device is read only')
        self.__dict__[key] = value

    def __str__(self):
        output = ''
        output += '%s\t(type:%s, model:%s, serial:%s)\t %s \n' % (self.name, self._type, self.model, self.serial, self.status)
        output += '['
        for attributeName, attributeValue in self._attribute.items():
            output += '%s: %s, ' %(attributeName, attributeValue)
        output = output[:-2]
        output += ']'
        return output

    # def __repr__(self):
    #     return self.__str__()

    def init(self, apiDict):
        self.serial = apiDict['device_sn']
        self.name = apiDict['device_name']
        self.model = apiDict['device_model']
        self.status = DEVICE_STATE(apiDict['status'])
        self._type = DEVICE_TYPE(apiDict['device_type'])
        self._callbacks = {}
        for attribute in apiDict['params']:
            try:
                self._attribute[PARAM_TYPE(attribute['param_type'])] = attribute['param_value']
            except:
                self._notRecognizedAttribute[attribute['param_type']] = attribute['param_value']
                self._logger.debug('%s: attribute type \'%s\' is not recognized: %s' % (self.model, attribute['param_type'], attribute['param_value']))
        
        for key in apiDict.keys():
            if key not in EXCLUDED_ROOT_PROPERTY:
                if(key == 'wifi_mac' and ':' not in apiDict[key]):
                    new_mac = ''
                    for i in range(0,len(apiDict[key]),2):
                        new_mac += apiDict[key][i:i+2] + ':'
                    apiDict[key] = new_mac[:-1]
                try:
                    self._attribute[PARAM_TYPE(key)] = apiDict[key]
                except:
                    self.__dict__[key] = apiDict[key]
                    pass
        # self.deviceType = DEVICE_TYPE(apiDict['device_type'])
    
    def update(self, apiDict):
        updatedAttributes = []
        for key in apiDict.keys():
            if key not in EXCLUDED_ROOT_PROPERTY:
                if(key in self.__dict__ and self.__dict__[key] != apiDict[key]):
                    self._logger.info('%s updated %s: %s -> %s' %(self.name, key, self.__dict__[key], apiDict[key]))
                    self.__dict__[key] = apiDict[key]
                try:
                    if(PARAM_TYPE(key) in self._attribute and self._attribute[PARAM_TYPE(key)] != apiDict[key]):
                        self._logger.info('%s updated %s: %s -> %s' %(self.name, PARAM_TYPE(key), self._attribute[PARAM_TYPE(key)], apiDict[key]))
                        updatedAttributes.append((PARAM_TYPE(key), self._attribute[PARAM_TYPE(key)], apiDict[key]))
                        self._attribute[PARAM_TYPE(key)] = apiDict[key]
                    elif(PARAM_TYPE(key) not in self._attribute):
                        self._logger.info('%s new attribute %s: %s -> %s' %(self.name, PARAM_TYPE(key), self._attribute[PARAM_TYPE(key)], apiDict[key]))
                        updatedAttributes.append((PARAM_TYPE(key), self._attribute[PARAM_TYPE(key)], apiDict[key]))
                        self._attribute[PARAM_TYPE(key)] = apiDict[key]
                except:
                    pass
                
        for attribute in apiDict['params']:
            try:
                if(PARAM_TYPE(attribute['param_type']) in self._attribute and self._attribute[PARAM_TYPE(attribute['param_type'])] != attribute['param_value']):
                    updatedAttributes.append((PARAM_TYPE(attribute['param_type']), self._attribute[PARAM_TYPE(attribute['param_type'])], attribute['param_value']))
                    self._logger.info('%s updated %s: %s -> %s' %(self.name, PARAM_TYPE(attribute['param_type']), self._attribute[PARAM_TYPE(attribute['param_type'])], attribute['param_value']))
                    self._attribute[PARAM_TYPE(attribute['param_type'])] = attribute['param_value']
                elif(PARAM_TYPE(attribute['param_type']) not in self._attribute):
                    updatedAttributes.append((PARAM_TYPE(attribute['param_type']), self._attribute[PARAM_TYPE(attribute['param_type'])], attribute['param_value']))
                    self._attribute[PARAM_TYPE(attribute['param_type'])] = attribute['param_value']
                    self._logger.info('%s new attribute %s: %s' % (self.name, PARAM_TYPE(attribute['param_type']), attribute['param_value']))
            except:
                if(attribute['param_type'] in self._notRecognizedAttribute and self._notRecognizedAttribute[attribute['param_type']] != attribute['param_value']):
                    self._logger.info('%s updated %s: %s -> %s' %(self.name, attribute['param_type'], self._notRecognizedAttribute[attribute['param_type']], attribute['param_value']))
                    self._notRecognizedAttribute[attribute['param_type']] = attribute['param_value']
                elif(attribute['param_type'] not in self._notRecognizedAttribute):
                    self._notRecognizedAttribute[attribute['param_type']] = attribute['param_value']
                    self._logger.info('%s new attribute %s: %s' % (self.name, attribute['param_type'], attribute['param_value']))

        if(len(updatedAttributes) > 0):
            self._call(updatedAttributes)
    
    def _call(self, updatedAttributes):
        for cId in self._callbacks:
            data = []
            for attributeTuple in updatedAttributes:
                if (attributeTuple[0] in self._callbacks[cId].attributes or len(self._callbacks[cId].attributes) == 0):
                    data.append(attributeTuple)
            if(len(data) > 0):
                asyncio.create_task(self._callbacks[cId].call(data))
        pass
    
    def subscribe(self, attributes: [PARAM_TYPE], callback):
        cId = getUniqueId(callback)
        if(cId in self._callbacks):
            self._callbacks[cId].attributes = list(set(self._callbacks[cId].attributes + attributes))
        else:
            self._callbacks[cId] = callbackStruct(cId, attributes, callback)
        return 'OK'

    def unsubscribe(self, callback):
        cId = getUniqueId(callback)
        if(cId in self._callbacks):
            del self._callbacks[cId]
            return 'OK'
        else:
            return 'KO'
    
    @classmethod
    def fromType(cls, api, deviceType: DEVICE_TYPE):
        if cls.__name__ == 'Device':
            for subclass in cls.__subclasses__():
                if deviceType in subclass.deviceType:
                    return subclass(api)
            return Device(api)
        else:
            return Device.fromType(api, deviceType)
        pass

    @property
    def attribute(self):
        return self._attribute

    @property
    def state(self):
        if(PARAM_TYPE.PROP_STATUS in self._attribute):
            return self._attribute[PARAM_TYPE.PROP_STATUS].name.replace('_', ' ').lower()
        else:
            return self.status.name.replace('_', ' ').lower()

    @property
    def hasbattery(self):
        return PARAM_TYPE.BATTERY_LEVEL in self._attribute

    @property
    def battery_level(self):
        return self._attribute[PARAM_TYPE.BATTERY_LEVEL] if self.hasbattery else None
    
    @property
    def isCamera(self):
        return self._type in [
            DEVICE_TYPE.CAMERA_E,
            DEVICE_TYPE.CAMERA,
            DEVICE_TYPE.CAMERA2_PRO,
            DEVICE_TYPE.CAMERA2C_PRO,
            DEVICE_TYPE.CAMERA2C,
            DEVICE_TYPE.CAMERA2,
            DEVICE_TYPE.INDOOR_CAMERA_1080,
            DEVICE_TYPE.INDOOR_PT_CAMERA_1080,
            DEVICE_TYPE.SOLO_CAMERA_PRO,
            DEVICE_TYPE.SOLO_CAMERA,
            DEVICE_TYPE.DOORBELL,
            DEVICE_TYPE.BATTERY_DOORBELL,
            DEVICE_TYPE.BATTERY_DOORBELL_2,
            DEVICE_TYPE.FLOODLIGHT
        ]
    
    @property
    def isMotionSensor(self):
        return self._type in [
            DEVICE_TYPE.MOTION_SENSOR
        ] or self.isCamera
    
    @property
    def isDoorLock(self):
        return self._type in [
            DEVICE_TYPE.LOCK_BASIC,
            DEVICE_TYPE.LOCK_ADVANCED,
            DEVICE_TYPE.LOCK_BASIC_NO_FINGER,
            DEVICE_TYPE.LOCK_ADVANCED_NO_FINGER
        ]


class Station(Device):
    deviceType= [DEVICE_TYPE.STATION]

    def init(self, apiDict):
        self.serial = apiDict['station_sn']
        self.name = apiDict['station_name']
        self.model = apiDict['station_model']
        self.status = DEVICE_STATE(apiDict['status'])
        self._type = Station.deviceType
        for attribute in apiDict['params']:
            try:
                self._attribute[PARAM_TYPE(attribute['param_type'])] = attribute['param_value']
            except:
                self._notRecognizedAttribute[attribute['param_type']] = attribute['param_value']
                self._logger.debug('%s: attribute type \'%s\' is not recognized: %s' % (self.model, attribute['param_type'], attribute['param_value']))        
        for key in apiDict.keys():
            if key not in EXCLUDED_ROOT_PROPERTY:
                try:
                    self._attribute[PARAM_TYPE(key)] = apiDict[key]
                except:
                    self.__dict__[key] = apiDict[key]
                    pass
        # self.deviceType = DEVICE_TYPE(apiDict['device_type'])


class MotionSensor(Device):
    deviceType = [DEVICE_TYPE.MOTION_SENSOR]

    @property
    def motionDetected(self):
        lowerWindowLimit = datetime.now()-timedelta(milliseconds=MOTION_DETECTION_COOLDOWN_MS)
        return  lowerWindowLimit <= datetime.fromtimestamp(self._attribute[PARAM_TYPE.PROP_UPDATE_TIME])

class GenericCamera(Device):
    deviceType = [
        DEVICE_TYPE.CAMERA_E,
        DEVICE_TYPE.CAMERA,
        DEVICE_TYPE.CAMERA2_PRO,
        DEVICE_TYPE.CAMERA2C_PRO,
        DEVICE_TYPE.CAMERA2C,
        DEVICE_TYPE.CAMERA2,
        DEVICE_TYPE.INDOOR_CAMERA_1080,
        DEVICE_TYPE.INDOOR_PT_CAMERA_1080,
        DEVICE_TYPE.SOLO_CAMERA_PRO,
        DEVICE_TYPE.SOLO_CAMERA,
        DEVICE_TYPE.DOORBELL,
        DEVICE_TYPE.BATTERY_DOORBELL,
        DEVICE_TYPE.BATTERY_DOORBELL_2,
        DEVICE_TYPE.FLOODLIGHT
    ]
    
    @property
    def motionDetected(self):
        lowerWindowLimit = datetime.now()-timedelta(milliseconds=MOTION_DETECTION_COOLDOWN_MS)
        return  lowerWindowLimit <= datetime.fromtimestamp(self._attribute[PARAM_TYPE.PROP_UPDATE_TIME])


import click, logging, asyncio, time
from .api import Api
import asyncio

_logger = logging.getLogger(__name__)
logging.basicConfig(format='%(name)-10s %(levelname)-8s %(message)s')

uefyApi = None


#pylint: disable=no-value-for-parameter
try:
    loop = asyncio.get_running_loop()
except:
    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)

class MyContext(click.Context):
    def __init__(self):
        self.uefyApi = None

@click.group(chain=True)
@click.pass_context
@click.option('--debug/--no-debug', prompt='activate debug', default=False)
def cli(ctx, debug):
    _logger.debug('cli: %s, %s' %(type(ctx), ctx))
    if(ctx.obj is None):
        ctx.obj = MyContext()
    _logger.parent.setLevel(logging.DEBUG if debug else logging.INFO)
    _logger.setLevel(logging.DEBUG if debug else logging.INFO)

@cli.command()
@click.pass_context
@click.option('--username', prompt='eufy security username', default=None)
@click.option('--password', prompt='eufy security password', default=None)
@click.option('--tfa', prompt='preferred two factor authentication method (EMAIL, SMS, PUSH)', default='EMAIL')
def login(ctx, username, password, tfa):
    _logger.debug('Username: %s, Password: %s, tfa: %s' % (username, password, tfa))
    ctx.parent.obj.uefyApi = Api(username=username, password=password)
    try:
        asyncio.run(ctx.parent.obj.uefyApi.authenticate())
        click.echo('Token: %s' % ctx.parent.obj.uefyApi.token)
        click.echo('Token_expire: %s' % ctx.parent.obj.uefyApi.token_expire_at)
        click.echo('Domain: %s' % ctx.parent.obj.uefyApi.domain)
    except:
        pass
    pass

@cli.command()
@click.pass_context
@click.option('--token', prompt='eufy token', default=None, required=False)
@click.option('--token_expire_at', prompt='eufy token expiratio timestamp', default=None, required=False)
@click.option('--domain', prompt='eufy domain', default=None, required=False)
def session(ctx, token, domain, token_expire_at):
    ctx.parent.obj.uefyApi = Api(token=token, domain=domain, token_expire_at=int(token_expire_at))

@cli.command()
@click.pass_context
def devices(ctx):
    try:
        asyncio.run(ctx.parent.obj.uefyApi.update())
        for deviceSn, device in ctx.parent.obj.uefyApi.devices.items():
            click.echo(device)
        for stationSn, station in ctx.parent.obj.uefyApi.stations.items():
            click.echo(station)
        # print(ctx.parent.obj.uefyApi.devices.values())
    except Exception as e:
        _logger.exception(e)
    pass

@cli.command()
@click.pass_context
@click.option('--serial', prompt='eufy device serial id', default=None, required=False)
def monitor(ctx, serial):
    asyncio.run(ctx.parent.obj.uefyApi.update(device_sn=serial))
    async def update_print(attributes):
        for attribute in attributes:
            # _logger.debug(attribute)
            click.echo('updated: %s, from \'%s\' to \'%s\'' % attribute)
    if(serial in ctx.parent.obj.uefyApi.devices):
        ctx.parent.obj.uefyApi.devices[serial].subscribe([], update_print)
        try:
            for i in range(0,60*5):
                asyncio.run(ctx.parent.obj.uefyApi.update(device_sn=serial))
                time.sleep(1)
        except Exception as e:
            _logger.exception(e)
    else:
        click.echo('unknown serial: %s' % serial)

if __name__ == '__main__':
    cli()
    loop.run_until_complete()
    loop.close()


from eufySecurityApi.const import TWO_FACTOR_AUTH_METHODS, API_BASE_URL, API_HEADERS, RESPONSE_ERROR_CODE, ENDPOINT_LOGIN,ENDPOINT_DEVICE_LIST, DEVICE_TYPE, ENDPOINT_STATION_LIST

import logging, json, copy, functools, requests, asyncio
from datetime import datetime, timedelta #, time as dtTime
from eufySecurityApi.model import Device
# import time

_LOGGER = logging.getLogger(__name__)
class Api():
    def __init__(self, username=None, password=None, token=None, domain=API_BASE_URL, token_expire_at=None, preferred2FAMethod=TWO_FACTOR_AUTH_METHODS.EMAIL):
        self._username =username
        self._password = password
        self._preferred2FAMethod = preferred2FAMethod
        self._token = token
        self._tokenExpiration = None if token_expire_at is None else datetime.fromtimestamp(token_expire_at)
        self._refreshToken = None
        self._domain = domain
        self.headers = API_HEADERS
        self._LOGGER = logging.getLogger(__name__)
        self.devices = {}
        self.stations = {}
        self._userId = None
        # self.headers['timezone'] = 
        #    dtTime(dtTime.fromisoformat(time.strptime(time.localtime(), '%HH:%MM'))) - dtTime(dtTime.fromisoformat(time.strptime(time.gmtime(), '%HH:%MM')))
    
    @property
    def userId(self):
        return self._userId
    async def authenticate(self):
        if(self._token is None or self._tokenExpiration > datetime.now()):
            response = await self._request('POST', ENDPOINT_LOGIN, {
                'email': self._username,
                'password': self._password
            }, self.headers)
            if(response.status_code != 200):
                self._LOGGER.error('Unexpected response code: %s, on url: %s' % response.status_code, response.request.url)
                raise LoginException('Unexpected response code: %s, on url: %s' % response.status_code, response.request.url)
            dataresult = response.json()
            self._LOGGER.debug('login response: %s' % dataresult)
            # self._LOGGER.debug('%s, %s' % (type(dataresult['code']), dataresult['code']))
            if(RESPONSE_ERROR_CODE(dataresult['code']) == RESPONSE_ERROR_CODE.WHATEVER_ERROR):
                self._token = dataresult['data']['auth_token']
                self._tokenExpiration = datetime.fromtimestamp(dataresult['data']['token_expires_at'])
                if('domain' in dataresult['data'] and dataresult['data']['domain'] != '' and dataresult['data']['domain'] != self._domain):
                    self._token = None
                    self._tokenExpiration = None
                    self._domain = dataresult['data']['domain']
                    self._LOGGER.info('Switching to new domain: %s', self._domain)
                    return await self.authenticate()

                self._LOGGER.debug('Token: %s' %self._token)
                self._LOGGER.debug('Token expire at: %s' % self._tokenExpiration)
                self._userId = dataresult['data']['user_id']
                return 'OK'
            elif(RESPONSE_ERROR_CODE(dataresult['code']) == RESPONSE_ERROR_CODE.NEED_VERIFY_CODE):
                self._LOGGER.info('need two factor authentication. Send verification code...')
                #dataresult['data']
                self._token = dataresult['data']['auth_token']
                self._tokenExpiration = datetime.fromtimestamp(dataresult['data']['token_expires_at'])
                self._userId = dataresult['data']['user_id']
                self._LOGGER.debug('Token: %s' %self._token)
                self._LOGGER.debug('Token expire at: %s' % self._tokenExpiration)

                await self.send_verify_code()
                return "send_verify_code"
            else:
                message = 'Unexpected API response code %s: %s (%s)' % (dataresult['code'], dataresult['msg'], response.request.url)
                self._LOGGER.error(message)
                raise LoginException(message)
        else:
            return 'OK'
        pass

    async def update(self, device_sn=None):
        if(device_sn is None):
            await self.get_stations()
            await self.get_devices()
        else:
            await self.get_devices(device_sn)


    async def get_devices(self, device_sn=None):
        data = {}
        if(device_sn is not None):
            data['device_sn'] = device_sn
        response = await self._request('POST', ENDPOINT_DEVICE_LIST, data, self.headers)
        if(response.status_code != 200):
            self._LOGGER.error('Unexpected response code: %s, on url: %s' % (response.status_code, response.request.url))
            raise LoginException('Unexpected response code: %s, on url: %s' % (response.status_code, response.request.url))
        dataresult = response.json()
        self._LOGGER.debug('get_devices response: %s' % dataresult)
        if(RESPONSE_ERROR_CODE(dataresult['code']) != RESPONSE_ERROR_CODE.WHATEVER_ERROR):
            message = 'Unexpected API response code %s: %s' % (dataresult['code'], dataresult['msg'])
            self._LOGGER.error(message)
            raise ApiException(message)
        for device in dataresult['data']:
            try:
                deviceType = DEVICE_TYPE(device['device_type'])
                if(device['device_sn'] not in self.devices):
                    self.devices[device['device_sn']] = Device.fromType(self, deviceType)
                    self.devices[device['device_sn']].init(device)
                else:
                    self.devices[device['device_sn']].update(device)
            except Exception as e:
                self._LOGGER.exception(e)
        return self.devices

    async def get_stations(self):
        response = await self._request('POST', ENDPOINT_STATION_LIST, {}, self.headers)
        if(response.status_code != 200):
            self._LOGGER.error('Unexpected response code: %s, on url: %s' % (response.status_code, response.request.url))
            raise LoginException('Unexpected response code: %s, on url: %s' % (response.status_code, response.request.url))
        dataresult = response.json()
        self._LOGGER.debug('get_stations response: %s' % dataresult)
        if(RESPONSE_ERROR_CODE(dataresult['code']) != RESPONSE_ERROR_CODE.WHATEVER_ERROR):
            message = 'Unexpected API response code %s: %s' % (dataresult['code'], dataresult['msg'])
            self._LOGGER.error(message)
            raise ApiException(message)
        for device in dataresult['data']:
            try:
                deviceType = DEVICE_TYPE(device['device_type'])
                self.stations[device['station_sn']] = Device.fromType(self, deviceType)
                self.stations[device['station_sn']].init(device)
            except Exception as e:
                self._LOGGER.exception(e)
        return self.stations

    async def get_device(self, deviceId):
        pass

    async def refresh_token(self):
        pass

    async def invalidate_token(self):
        self._token = None
        self._refreshToken = None
        self._tokenExpiration = None
        pass

    async def send_verify_code(self):
        response = await self._request('POST', 'sms/send/verify_code', {
            'message_type': self._preferred2FAMethod
        }, self.headers)
        if(response.status_code != 200):
            self._LOGGER.error('Unexpected response code: %s, on url: %s' % response.status_code, response.request.url)
            raise ApiException('Unexpected response code: %s, on url: %s' % response.status_code, response.request.url)
        dataresult = response.json()
        self._LOGGER.debug('login response: %s' % dataresult)
        if(RESPONSE_ERROR_CODE(dataresult['code']) != RESPONSE_ERROR_CODE.WHATEVER_ERROR):
            message = 'Unexpected API response code %s: %s' % (dataresult['code'], dataresult['msg'])
            self._LOGGER.error(message)
            raise ApiException(message)
        return 'OK'

    @property
    def connected(self):
        return self._token != None
    
    @property
    def base_url(self):
        return ('https://%s/v1' % self._domain)

    @property 
    def token(self):
        return self._token

    @property 
    def token_expire_at(self):
        return self._tokenExpiration.timestamp()
    @property 
    def domain(self):
        return self._domain
    async def _request(self, method, url, data, headers={}) -> requests.Response:
        try:
            loop = asyncio.get_running_loop()
        except:
            loop = asyncio.new_event_loop()
            asyncio.set_event_loop(loop)
        call = None
        if(method == 'GET'):
            call = requests.get
        elif(method == 'POST'):
            call = requests.post
        else:
            raise ApiException('Unsupported operation: %s' % method)
        url = self.base_url + url
        
        newHeaders = copy.copy(headers)
        if(url != ENDPOINT_LOGIN):
            newHeaders['X-Auth-Token'] = self._token
        
        
        self._LOGGER.debug('method: %s' % method)
        self._LOGGER.debug('url: %s' % url)
        self._LOGGER.debug('data: %s' % data)
        self._LOGGER.debug('headers: %s' % newHeaders)
        response = await loop.run_in_executor(None, functools.partial(call, url, json=data, headers=newHeaders))
        #response = call(url, json=data, headers=newHeaders)
        return response



class ApiException(Exception):
    pass

class LoginException(ApiException):
    pass


import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="eufy-security-api-rihan", # Replace with your own username
    version="0.0.15",
    author="Rihan",
    author_email="aleatza@gmail.com",
    description="Cover the http API for eufy security devices",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Rihan9/eufySecurityApi",
    packages=setuptools.find_packages(),
    install_requires= ['requests', 'click', 'pytz', 'setuptools', 'asyncio'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    license= 'MIT License'
)